@@ -1,9 +1,5 @@
 import ast
 import sys
-if sys.version_info[0] < 3:
-    from urlparse import urlparse
-else:
-    from urllib.parse import urlparse
 
 
 input_filenames_argument_name = 'input_filenames'
@@ -231,8 +227,7 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         return node
 
     def visit_Attribute(self, node):
-        value_rep = self.get_rep(node.value)
-        node.rep = value_rep + '[' + repr(node.attr) + ']'
+        node.rep = self.get_rep(node.value) + '.' + node.attr
         return node
 
     def visit_Lambda(self, node):
@@ -269,11 +264,11 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
                     urls = node.args[0].elts
                 else:
                     urls = [node.args[0]]
-                paths = [''.join(urlparse(ast.literal_eval(url))[1:])
-                         for url in urls if ast.literal_eval(url) is not None]
+                urls = [ast.literal_eval(url) for url in urls]
+                urls = [url for url in urls if url is not None]
                 source_rep = (input_filenames_argument_name + ' '
                               + 'if ' + input_filenames_argument_name + ' is not None '
-                              + 'else ' + repr(paths))
+                              + 'else ' + repr(urls))
             else:
                 source_rep = input_filenames_argument_name
             source_rep = ('(lambda source: [source] if isinstance(source, str) else source)('
@@ -298,7 +293,11 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
                         + ' for input_file in input_files}))[1])'
                         + '(' + source_rep + ', ' + tree_name_rep + ')')
         else:
-            func_rep = self.get_rep(node.func)
+            if isinstance(node.func, ast.Attribute) and node.func.attr == 'ToFourMomenta':
+                func_rep = 'vector.awk'
+                node.args.append(node.func.value)
+            else:
+                func_rep = self.get_rep(node.func)
             args_rep = ', '.join(self.get_rep(arg) for arg in node.args)
             node.rep = func_rep + '(' + args_rep + ')'
         return node
@@ -366,3 +365,38 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         self.visit(node.source)
         node.rep = 'ak.num(' + self.get_rep(node.source) + ', axis=' + repr(self._depth) + ')'
         return node
+
+    def visit_Choose(self, node):
+        self.visit(node.source)
+        node.rep = ('ak.combinations(' + self.get_rep(node.source) + ', ' + self.get_rep(node.n)
+                    + ', axis=' + repr(self._depth) + ')')
+        return node
+
+    def visit_OrderBy(self, node):
+        if type(node.key_selector) is not ast.Lambda:
+            raise TypeError('Argument to OrderBy() must be a lambda function, found '
+                            + node.key_selector)
+        if len(node.key_selector.args.args) != 1:
+            raise TypeError('Lambda function in OrderBy() must have exactly one argument, found '
+                            + len(node.key_selector.args.args))
+        self.visit(node.source)
+        self._depth += 1
+        if sys.version_info[0] < 3:
+            subscriptable = node.key_selector.args.args[0].id
+        else:
+            subscriptable = node.key_selector.args.args[0].arg
+        self.visit(node.key_selector)
+        delattr(node.key_selector, 'rep')
+        node.key_selector.body.rep = ('ak.argsort(' + self.get_rep(node.key_selector.body)
+                                      + ', axis=' + repr(self._depth - 1) + ')')
+        if sys.version_info[0] < 3 or (sys.version_info[0] == 3 and sys.version_info[1] < 9):
+            slice_node = ast.Index(node.key_selector.body)
+        else:
+            slice_node = node.key_selector.body
+        node.key_selector.body = ast.Subscript(value=ast.Name(id=subscriptable),
+                                               slice=slice_node,
+                                               short_circuit=True)
+        call_node = ast.Call(func=node.key_selector, args=[node.source])
+        node.rep = self.get_rep(call_node)
+        self._depth -= 1
+        return node
